#! /usr/bin/python3
# -*- coding: utf-8 -*-

'''
unity_remmina_daemon.py parses Remmina saved clients for the Boxes master scope
'''

# Copyright(C) 2013 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import Unity
from gi.repository import Gio, GLib
import gettext
import os
import configparser

APP_NAME = 'unity-scope-remmina'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Boxes.Remmina'
UNIQUE_PATH = '/com/canonical/unity/scope/boxes/remmina'
SEARCH_URI = ''
SEARCH_HINT = _('Search Remmina')
NO_RESULTS_HINT = _('Sorry, there are no Remmina results that match your search.')
PROVIDER_CREDITS = _('Powered by Remmina')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-remmina.svg'
DEFAULT_RESULT_ICON = 'remmina'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT
REMMINA_SETTINGS_PATH = os.getenv("HOME") + '/.remmina/'
REMMINA_EXECUTABLE = '/usr/bin/remmina'
PROTOCOLS = {'SSH': _('SSH: Secure Shell'),
             'VNC': _('VNC: Virtual Network Computing'),
             'SFTP': _('SFTP: Secure File Transfer Protocol'),
             'RDP': _('RDP: Remote Desktop Protocol'),
             'VNCI': _('VNC: Incoming Connection'),
             'NX': _('NX: NX Technology'),
             'XDMCP': _('XDMCP: X Remote Session')}

CAT0 = {'id': 'boxes',
        'name': _('Computers'),
        'icon': SVG_DIR + 'group-installed.svg',
        'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [CAT0]

FILTERS = []

META0 = {'id': 'protocol',
         'type': 's',
         'field': Unity.SchemaFieldType.OPTIONAL}
META1 = {'id': 'server',
         'type': 's',
         'field': Unity.SchemaFieldType.OPTIONAL}
EXTRA_METADATA = [META0, META1]


def search(query, filters):
    '''
    Any search method returning results as a list of tuples.
    Available tuple fields:
    uri(string)
    icon(string)
    title(string)
    comment(string)
    dnd_uri(string)
    mimetype(string)
    category(int)
    result_type(Unity ResultType)
    extras metadata fields(variant)
    '''
    results = []
    if len(query) < 2:
        return results

    for prefs_file in os.listdir(REMMINA_SETTINGS_PATH):
        if prefs_file.endswith('.remmina'):
            config = configparser.ConfigParser(allow_no_value=True)
            config.read(REMMINA_SETTINGS_PATH + prefs_file)
            title = config.get('remmina', 'name')
            server = config.get('remmina', 'server')
            protocol = config.get('remmina', 'protocol')
            if query.lower() in title.lower() or query.lower() in server.lower():
                results.append({'uri': REMMINA_SETTINGS_PATH + prefs_file,
                                'title': title,
                                'protocol': protocol,
                                'server': server})
    return results


class Preview(Unity.ResultPreviewer):
    '''
    Creates the preview for the result
    '''
    def do_run(self):
        '''
        Create a preview and return it
        '''
        image = Gio.ThemedIcon.new(DEFAULT_RESULT_ICON)
        preview = Unity.GenericPreview.new(self.result.title, '', image)
        preview.props.description_markup = self.result.comment
        try:
            preview.add_info(Unity.InfoHint.new('protocol', _("Protocol"), None, PROTOCOLS[self.result.metadata['protocol'].get_string()]))
        except KeyError:
            preview.add_info(Unity.InfoHint.new('protocol', _("Protocol"), None, self.result.metadata['protocol'].get_string()))
        preview.add_info(Unity.InfoHint.new('server', _("Server"), None, self.result.metadata['server'].get_string()))
        show_action = Unity.PreviewAction.new("connect", _("Connect"), None)
        preview.add_action(show_action)
        return preview

# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    '''
    The MySearch() class returns the results to Scope()
    '''
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result_set.add_result(**i)
        except Exception as error:
            print('Error in MySearch(): %s' % error)


class Scope(Unity.AbstractScope):
    '''
    The Scope() class contains all the scope's details for unity to display
    '''
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        '''
        Returns the SEARCH_HINT text
        '''
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for metadata in EXTRA_METADATA:
                schema.add_field(metadata['id'], metadata['type'], metadata['field'])
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        category_set = Unity.CategorySet.new()
        if CATEGORIES:
            for cat in CATEGORIES:
                category = Unity.Category.new(cat['id'],
                                              cat['name'],
                                              Gio.ThemedIcon.new(cat['icon']),
                                              cat['renderer'])
                category_set.add(category)
        return category_set

    def do_get_filters(self):
        '''
        Adds filters
        '''
        filters = Unity.FilterSet.new()
#        if FILTERS:
#
        return filters

    def do_get_group_name(self):
        '''
        Returns GROUP_NAME
        '''
        return GROUP_NAME

    def do_get_unique_name(self):
        '''
        Returns UNIQUE_PATH
        '''
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        '''
        Creates the search
        '''
        scope_search = MySearch(search_context)
        return scope_search

    def do_activate(self, result, metadata, action):
        '''
        What to do when a resut is clicked
        '''
        parameters = [REMMINA_EXECUTABLE, '-c', result.uri]
        GLib.spawn_async(parameters)
        return Unity.ActivationResponse(handled=Unity.HandledType.HIDE_DASH, goto_uri=None)

    def do_create_previewer(self, result, metadata):
        '''
        Creates a preview when a resut is right-clicked
        '''
        result_preview = Preview()
        result_preview.set_scope_result(result)
        result_preview.set_search_metadata(metadata)
        return result_preview


def load_scope():
    '''
    Called by scope-runner.py and returns the scope
    '''
    return Scope()
